﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Marketing;
using Vevo.Shared.DataAccess;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class BannerRepositoryNoCache : IBannerRepository
    {
        #region Private

        private const string bannerFields = " BannerID, StoreID, Name, CreateDate, EndDate, IsEnabled, SortOrder ";
        private const string bannerLocaleFields = " BannerID, CultureID, ImageURL, LinkURL, Description ";
        private const string allFields = " Banner.BannerID, StoreID, Name, CreateDate, EndDate, IsEnabled, SortOrder, " +
                                         " CultureID, ImageURL, LinkURL, Description ";

        private string CreateSortByText( string sortBy )
        {
            if ( sortBy == null || sortBy.Trim() == "" )
                return "";
            else if ( sortBy.ToLower().StartsWith( "bannerid" ) )
                return "ORDER BY Banner." + sortBy;
            else
                return "ORDER BY " + sortBy;
        }

        private string ConvertFieldName( string fieldName )
        {
            switch ( fieldName.ToLower() )
            {
                case "bannerid":
                    return "Banner.BannerID";
                default:
                    return fieldName;
            }
        }

        private Banner CreateOneFromRow( Culture culture, DataRow row )
        {
            Banner banner = new Banner( culture );

            banner.BannerID = row[ "BannerID" ].ToString();
            banner.StoreID = row[ "StoreID" ].ToString();
            banner.Name = row[ "Name" ].ToString();
            banner.CreateDate = Convert.ToDateTime( row[ "CreateDate" ] );
            banner.EndDate = Convert.ToDateTime( row[ "EndDate" ] );
            banner.IsEnabled = Convert.ToBoolean( row[ "IsEnabled" ] );
            banner.SortOrder = Convert.ToInt32( row[ "SortOrder" ] );

            return banner;
        }

        private BannerLocale CreateOneLocaleFromRow( DataRow row )
        {
            BannerLocale locale = new BannerLocale();
            locale.CultureID = row[ "CultureID" ].ToString();
            locale.ImageURL = row[ "ImageURL" ].ToString();
            locale.LinkURL = row[ "LinkURL" ].ToString();
            locale.Description = row[ "Description" ].ToString();

            return locale;
        }

        private IList<Banner> ConvertTableToList( Culture culture, DataTable table )
        {
            return ConvertTableToList( culture, table, 0, table.Rows.Count );
        }

        private IList<Banner> ConvertTableToList( Culture culture, DataTable table, int startIndex, int endIndex )
        {
            IList<Banner> list = new List<Banner>();

            for ( int i = startIndex; i <= endIndex && i < table.Rows.Count; i++ )
                list.Add( CreateOneFromRow( culture, table.Rows[ i ] ) );

            return list;
        }

        private List<ILocale> ConvertLocaleTableToList( DataTable localeTable )
        {
            List<ILocale> list = new List<ILocale>();

            foreach ( DataRow row in localeTable.Rows )
                list.Add( CreateOneLocaleFromRow( row ) );

            return list;
        }

        private void LocalesSaveAll( string bannerID, LocaleCollection<BannerLocale> bannerLocales )
        {
            bannerLocales.Load();
            DeleteLocales( bannerID );
            InsertLocales( bannerID, bannerLocales );
        }

        private void InsertLocales( string bannerID, LocaleCollection<BannerLocale> bannerLocales )
        {
            foreach ( BannerLocale locale in bannerLocales )
            {
                DataAccess.ExecuteNonQuery(
                "INSERT INTO BannerLocale ( " + bannerLocaleFields + " ) " +
                "VALUES ( @BannerID, @CultureID, @ImageURL, @LinkURL, @Description ); ",
                DataAccess.CreateParameterString( bannerID ),
                DataAccess.CreateParameterString( locale.CultureID ),
                DataAccess.CreateParameterString( locale.ImageURL ),
                DataAccess.CreateParameterString( locale.LinkURL ),
                DataAccess.CreateParameterString( locale.Description ) );
            }
        }

        private Banner Create( Banner banner )
        {
            string bannerID = DataAccess.ExecuteScalar(
                "INSERT INTO Banner ( StoreID, Name, CreateDate, EndDate, IsEnabled, SortOrder ) " +
                "VALUES ( @StoreID, @Name, @CreateDate, @EndDate, @IsEnabled, @SortOrder ) " +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterString( banner.StoreID ),
                DataAccess.CreateParameterString( banner.Name ),
                DataAccess.CreateParameterDateTime( banner.CreateDate ),
                DataAccess.CreateParameterDateTime( banner.EndDate ),
                DataAccess.CreateParameterBool( banner.IsEnabled ),
                DataAccess.CreateParameterInt32( banner.SortOrder ) );

            banner.BannerID = bannerID;
            LocalesSaveAll( banner.BannerID, banner.Locales );

            return banner;
        }

        private Banner Update( Banner banner )
        {
            DataAccess.ExecuteNonQuery( "UPDATE Banner " +
                "SET StoreID = @StoreID, Name = @Name, CreateDate = @CreateDate, EndDate = @EndDate, IsEnabled = @IsEnabled, SortOrder = @SortOrder " +
                "WHERE BannerID = @BannerID; ",
                DataAccess.CreateParameterString( banner.StoreID ),
                DataAccess.CreateParameterString( banner.Name ),
                DataAccess.CreateParameterDateTime( banner.CreateDate ),
                DataAccess.CreateParameterDateTime( banner.EndDate ),
                DataAccess.CreateParameterBool( banner.IsEnabled ),
                DataAccess.CreateParameterInt32( banner.SortOrder ),
                DataAccess.CreateParameterString( banner.BannerID ) );

            LocalesSaveAll( banner.BannerID, banner.Locales );

            return banner;
        }

        private void DeleteBanner( string bannerID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM Banner " +
                "WHERE BannerID = @BannerID; ",
                DataAccess.CreateParameterString( bannerID ) );
        }

        private void DeleteLocales( string bannerID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM BannerLocale " +
                "WHERE BannerID = @BannerID; ",
                DataAccess.CreateParameterString( bannerID ) );
        }

        #endregion

        #region Public Methods

        public Banner Save( Banner banner )
        {
            Banner result;

            using ( DataTransactionScope transaction = new DataTransactionScope() )
            {
                if ( banner.BannerID == "0" )
                    result = Create( banner );
                else
                    result = Update( banner );

                transaction.Complete();
            }

            return result;
        }

        public void Delete( string bannerID )
        {
            using ( DataTransactionScope transaction = new DataTransactionScope() )
            {
                DeleteBanner( bannerID );
                DeleteLocales( bannerID );

                transaction.Complete();
            }
        }

        public Banner GetOneByID( Culture culture, string bannerID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + allFields +
                "FROM Banner LEFT JOIN ( " +
                "   SELECT * " +
                "   FROM BannerLocale " +
                "   WHERE CultureID = @CultureID ) AS BannerLocale ON Banner.BannerID = BannerLocale.BannerID " +
                "WHERE Banner.BannerID = @BannerID; ",
                DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ),
                DataAccess.CreateParameterString( bannerID ) );

            if ( table.Rows.Count > 0 )
            {
                Banner banner = CreateOneFromRow( culture, table.Rows[ 0 ] );
                return banner;
            }
            else
            {
                return Banner.Null;
            }
        }

        public IList<Banner> GetAllByStoreID( Culture culture, string storeID, BoolFilter isEnabled, string sortBy )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + allFields +
                "FROM Banner LEFT JOIN BannerLocale ON Banner.BannerID = BannerLocale.BannerID " +
                "WHERE StoreID = @StoreID AND CultureID = @CultureID AND CreateDate <= @CreateDate AND EndDate >= @EndDate " +
                FlagFilterHelper.CreateFlagFilterWithAnd( "IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.CreateParameterString( storeID ),
                DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ),
                DataAccess.CreateParameterDateTime( DateTime.Now ),
                DataAccess.CreateParameterDateTime( DateTime.Now )
                );

            IList<Banner> bannerList = new List<Banner>();
            foreach ( DataRow row in table.Rows )
            {
                bannerList.Add( CreateOneFromRow( culture, row ) );
            }

            return bannerList;
        }

        public IList<Banner> SearchBanner(
            Culture culture,
            string sortBy,
            string storeID,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems
            )
        {
            ArrayList parameterList = new ArrayList();
            parameterList.Add( DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ) );
            parameterList.Add( DataAccess.CreateParameterString( storeID ) );

            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithAnd(
                searchFilter, ConvertFieldName( searchFilter.FieldName ), parameterList );

            DataTable full = DataAccess.ExecuteSelect(
                "SELECT " + allFields +
                "FROM Banner LEFT JOIN ( " +
                "   SELECT * " +
                "   FROM BannerLocale " +
                "   WHERE CultureID = @CultureID ) AS BannerLocale ON Banner.BannerID = BannerLocale.BannerID " +
                "WHERE StoreID = @StoreID " +
                commandSearch +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.ConvertArrayListToDbParameters( parameterList )
                );

            howManyItems = full.Rows.Count;

            DataColumn[] keys = new DataColumn[] { full.Columns[ "BannerID" ] };
            full.PrimaryKey = keys;
            return ConvertTableToList( culture, full, startIndex, endIndex );
        }

        public void UpdateBannerSortOrder( string storeID, string[] sortedBanner )
        {
            int sortCount = 0;
            foreach ( string item in sortedBanner )
            {
                sortCount++;
                DataAccess.ExecuteNonQuery(
                    "UPDATE Banner SET SortOrder = @SortOrder WHERE StoreID = @StoreID AND BannerID = @BannerID;",
                    DataAccess.CreateParameterString( sortCount.ToString() ),
                    DataAccess.CreateParameterString( storeID ),
                    DataAccess.CreateParameterString( item ) );
            }
        }

        public IList<ILocale> GetLocales( string bannerID )
        {
            DataTable localeTable = DataAccess.ExecuteSelect(
               "SELECT " + bannerLocaleFields +
               "FROM BannerLocale " +
               "WHERE BannerID = @BannerID; ",
               DataAccess.CreateParameterString( bannerID ) );

            return ConvertLocaleTableToList( localeTable );
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT Banner.BannerID, Name, CreateDate, EndDate, IsEnabled, ImageURL, LinkURL, Description " +
                "FROM Banner LEFT JOIN ( " +
                "   SELECT * " +
                "   FROM BannerLocale " +
                "   WHERE CultureID = 1 ) AS BannerLocale ON Banner.BannerID = BannerLocale.BannerID " +
                "WHERE 1 = 0;" );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        #endregion
    }
}
