﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using Vevo.Data.Implementation;
using Vevo.DataAccessLib;
using Vevo.Domain;
using Vevo.Domain.Blogs;
using Vevo.Domain.DataInterfaces;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class BlogRepositoryNoCache : IBlogRepository
    {
        #region Private

        private const string AllFields = "Blog.BlogID, BlogTitle, UrlName, IsEnabled, ShortContent, BlogContent, Tags, MetaTitle, MetaKeyword, MetaDescription, Publisher, CreateDate, ImageFile ";

        private UrlNameAccessHelper _urlNameHelper = new UrlNameAccessHelper( "Blog", "BlogID" );

        private BlogStoreAccess _blogStoreAccess = new BlogStoreAccess();
        private BlogCategoryItemAccess _blogCategoryItemAccess = new BlogCategoryItemAccess();

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else if (sortBy.ToLower() == "blogid")
            {
                return "ORDER BY Blog." + sortBy + " ";
            }
            else
                return "ORDER BY " + sortBy + " ";
        }

        private Blog Create( Blog blog )
        {
            Blog result;

            result = CreateBaseData( blog );

            _blogStoreAccess.Create( result.BlogID, blog.StoreIDs );
            return result;
        }

        private Blog CreateBaseData( Blog blog )
        {
            string blogID = DataAccess.ExecuteScalar(
                "INSERT INTO Blog (BlogTitle, IsEnabled, ShortContent, BlogContent, Tags, MetaTitle, MetaKeyword, MetaDescription, Publisher, CreateDate, ImageFile ) " +
                "VALUES (@BlogTitle, @IsEnabled, @ShortContent, @BlogContent, @Tags, @MetaTitle, @MetaKeyword, @MetaDescription, @Publisher, @CreateDate, @ImageFile ) " +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterString( blog.BlogTitle ),
                DataAccess.CreateParameterBool( blog.IsEnabled ),
                DataAccess.CreateParameterString( blog.ShortContent ),
                DataAccess.CreateParameterString( blog.BlogContent ),
                DataAccess.CreateParameterString( blog.Tags ),
                DataAccess.CreateParameterString( blog.MetaTitle ),
                DataAccess.CreateParameterString( blog.MetaKeyword ),
                DataAccess.CreateParameterString( blog.MetaDescription ),
                DataAccess.CreateParameterString( blog.Publisher ),
                DataAccess.CreateParameterDateTime( blog.CreateDate ),
                DataAccess.CreateParameterString( blog.ImageFile )
                );

            blog.BlogID = blogID;

            blog.UrlName = _urlNameHelper.UpdateUrlName( blog.BlogID.ToString(), blog.BlogTitle );
            _blogCategoryItemAccess.SaveAll( blog.BlogID, blog.BlogCategoryIDs );
            return blog;
        }

        private Blog Update( Blog blog )
        {
            DataAccess.ExecuteNonQuery( "UPDATE Blog " +
                "SET BlogTitle = @BlogTitle, " +
                "IsEnabled = @IsEnabled, ShortContent = @ShortContent, BlogContent = @BlogContent, Tags = @Tags, " +
                "MetaTitle = @MetaTitle, MetaKeyword = @MetaKeyword, MetaDescription = @MetaDescription, " +
                "Publisher = @Publisher, CreateDate = @CreateDate, ImageFile = @ImageFile " +
                "WHERE BlogID = @BlogID; ",
                DataAccess.CreateParameterString( blog.BlogTitle ),
                DataAccess.CreateParameterBool( blog.IsEnabled ),
                DataAccess.CreateParameterString( blog.ShortContent ),
                DataAccess.CreateParameterString( blog.BlogContent ),
                DataAccess.CreateParameterString( blog.Tags ),
                DataAccess.CreateParameterString( blog.MetaTitle ),
                DataAccess.CreateParameterString( blog.MetaKeyword ),
                DataAccess.CreateParameterString( blog.MetaDescription ),
                DataAccess.CreateParameterString( blog.Publisher ),
                DataAccess.CreateParameterDateTime( blog.CreateDate ),
                DataAccess.CreateParameterString( blog.ImageFile ),
                DataAccess.CreateParameterString( blog.BlogID ) );

            _blogStoreAccess.SaveAll( blog.BlogID, blog.StoreIDs );

            blog.UrlName = _urlNameHelper.UpdateUrlName(
                blog.BlogID.ToString(),
                blog.BlogTitle );
            _blogCategoryItemAccess.SaveAll( blog.BlogID, blog.BlogCategoryIDs );

            return blog;
        }

        private Blog CreateOneFromRow( DataRow row )
        {
            Blog blog = new Blog();

            blog.BlogID = row["BlogID"].ToString();
            blog.BlogTitle = row["BlogTitle"].ToString();
            blog.IsEnabled = ConvertUtilities.ToBoolean( row["IsEnabled"] );
            blog.UrlName = row["UrlName"].ToString();
            blog.ShortContent = row["ShortContent"].ToString();
            blog.BlogContent = row["BlogContent"].ToString();
            blog.Tags = row["Tags"].ToString();
            blog.MetaTitle = row["MetaTitle"].ToString();
            blog.MetaKeyword = row["MetaKeyword"].ToString();
            blog.MetaDescription = row["MetaDescription"].ToString();
            blog.Publisher = row["Publisher"].ToString();
            blog.CreateDate = ConvertUtilities.ToDateTime( row["CreateDate"] );
            blog.StoreIDs = _blogStoreAccess.GetByBlogID( blog.BlogID );
            blog.ImageFile = row["ImageFile"].ToString();
            blog.BlogCategoryIDs = _blogCategoryItemAccess.GetBlogCategoryIDByBlogID( row["BlogID"].ToString() );
            return blog;
        }

        private IList<Blog> ConvertTableToList( DataTable table )
        {
            return ConvertTableToList( table, 0, table.Rows.Count );
        }

        private IList<Blog> ConvertTableToList( DataTable table, int startIndex, int endIndex )
        {
            IList<Blog> list = new List<Blog>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
            {
                list.Add( CreateOneFromRow( table.Rows[i] ) );
            }

            return list;
        }

        private string ConvertFieldName( string fieldName )
        {
            switch (fieldName.ToLower())
            {
                case "blogid":
                    return "Blog.BlogID";

                default:
                    return fieldName;
            }
        }

        private string ConditionSearchField(
            string searchTerm,
            string[] searchFieldList,
            ArrayList parameters )
        {
            if (!String.IsNullOrEmpty( searchTerm ))
            {
                string[] arKeys = StringUtilities.SplitString( searchTerm, new char[] { ' ', '@', ';', '\'' } );
                string query = "";
                for (int i = 0; i < arKeys.Length; i++)
                {
                    if (i == 0)
                    {
                        query = " AND (";
                    }
                    for (int j = 0; j < searchFieldList.Length; j++)
                    {
                        string strColumn = searchFieldList[j];
                        if (j == 0 && i == 0)
                            query += " ";
                        else
                            query += " OR ";

                        query += strColumn;

                        parameters.Add( DataAccess.CreateParameterString( "%" + arKeys[i] + "%" ) );
                        query += " LIKE @" + strColumn + i;
                    }
                    if (i == arKeys.Length - 1)
                        query += " ) ";
                }
                return query;
            }
            else
                return "";
        }

        private IList<Blog> ExecuteQuickSearch(
            string searchTerm,
            string[] searchFieldList,
            string storeID,
            int startIndex,
            int endIndex,
            out int howManyItems
            )
        {
            ArrayList parameters = new ArrayList();
            parameters.Add( DataAccess.CreateParameterString( storeID ) );
            parameters.Add( DataAccess.CreateParameterBool( true ) );

            string query = "SELECT ROW_NUMBER() over ( ORDER BY CreateDate DESC ) as rownum, * " +
                    " FROM ( " +
                    "    SELECT DISTINCT Blog.* " +
                    "    FROM Blog, BlogStore " +
                    "    WHERE Blog.BlogID = BlogStore.BlogID and StoreID = @StoreID and IsEnabled = @IsEnabled and CreateDate < GETDATE() " +
                         ConditionSearchField( searchTerm, searchFieldList, parameters ) +
                    ") AS Blog ";

            string extQuery = "with SearchResult as (" + query + ") " +
                    " select * from SearchResult where rownum between " + (startIndex + 1).ToString() + " and " + (endIndex + 1).ToString() + ";";

            DataTable table = DataAccess.ExecuteSelect( extQuery, DataAccess.ConvertArrayListToDbParameters( parameters ) );
            howManyItems = table.Rows.Count;

            return ConvertTableToList( table, startIndex, endIndex );
        }

        #endregion

        #region Aggregate Methods



        #endregion

        #region Public Methods

        public Blog Save( Blog blog )
        {
            Blog result;

            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                if (blog.BlogID == "0")
                    result = Create( blog );
                else
                    result = Update( blog );

                transaction.Complete();
            }

            return result;
        }

        public Blog GetOne( string blogID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM Blog " +
                "WHERE Blog.BlogID = @BlogID; ",
                DataAccess.CreateParameterString( blogID )
                );

            if (table.Rows.Count > 0)
            {
                Blog blog = CreateOneFromRow( table.Rows[0] );
                return blog;
            }
            else
                return Blog.Null;
        }

        public string GetBlogIDFromUrlName( string urlName )
        {
            string blogID = DataAccess.ExecuteScalar(
                "SELECT BlogID " +
                "FROM Blog " +
                "WHERE UrlName = @UrlName; ",
                DataAccess.CreateParameterString( urlName ) );

            return blogID;
        }

        public string GetIDByTitle( string blogTitle )
        {
            string blogID = DataAccess.ExecuteScalar(
                "SELECT BlogID " +
                "FROM Blog " +
                "WHERE BlogTitle = @BlogTitle; ",
                DataAccess.CreateParameterString( blogTitle ) );

            return blogID;
        }

        public void Delete( string blogID )
        {
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                DataAccess.ExecuteNonQuery(
                    "DELETE " +
                    "FROM Blog " +
                    "WHERE BlogID = @BlogID; ",
                    DataAccess.CreateParameterString( blogID ) );

                _blogStoreAccess.DeleteByBlogID( blogID );

                DataAccess.ExecuteNonQuery(
                    "DELETE " +
                    "FROM BlogComment " +
                    "WHERE BlogID = @BlogID; ",
                    DataAccess.CreateParameterString( blogID ) );

                _blogCategoryItemAccess.DeleteByBlogID( blogID );

                transaction.Complete();
            }
        }

        public IList<Blog> GetAll( string sortBy )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM Blog " +
                CreateSortByText( sortBy ) + "; " );

            return ConvertTableToList( table );

        }

        public IList<Blog> SearchBlog(
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems,
            string storeID,
            bool allDate,
            BoolFilter isEnabled
            )
        {
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithWhere(
                searchFilter, ConvertFieldName( searchFilter.FieldName ), parameterList );

            if (!String.IsNullOrEmpty( storeID ))
                commandSearch += " AND StoreID = " + storeID + " ";
            commandSearch += " AND Blog.BlogID = BlogStore.BlogID ";

            if (!allDate)
            {
                commandSearch += " AND CreateDate < GETDATE() ";
            }

            DataTable full = DataAccess.ExecuteSelect(
                "SELECT Distinct " + AllFields +
                "FROM Blog, BlogStore " +
                commandSearch +
                FlagFilterHelper.CreateFlagFilterWithAnd( "IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.ConvertArrayListToDbParameters( parameterList )
                );

            howManyItems = full.Rows.Count;

            DataColumn[] keys = new DataColumn[] { full.Columns["BlogID"] };
            full.PrimaryKey = keys;
            return ConvertTableToList( full, startIndex, endIndex );
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM Blog " +
                "WHERE 1 = 0; "
                );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        public DataTable GetArchiveList( string storeID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " +
                "Month( CreateDate ) AS CreateMonth, Year(CreateDate) AS CreateYear, COUNT(Blog.BlogID) AS NumberOfBlog " +
                "FROM Blog, BlogStore " +
                "WHERE Blog.BlogID = BlogStore.BlogID and StoreID = @StoreID and IsEnabled = @IsEnabled and CreateDate < GETDATE() " +
                "GROUP BY Month( CreateDate ), Year(CreateDate) " +
                "ORDER BY Year(CreateDate) DESC, Month( CreateDate ) DESC; ",
                DataAccess.CreateParameterString( storeID ),
                DataAccess.CreateParameterBool( true ) );

            return table;
        }

        public DataTable GetArchiveList( string storeID, string limitRow )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT TOP " + limitRow + " " +
                "Month( CreateDate ) AS CreateMonth, Year(CreateDate) AS CreateYear, COUNT(Blog.BlogID) AS NumberOfBlog " +
                "FROM Blog, BlogStore " +
                "WHERE Blog.BlogID = BlogStore.BlogID and StoreID = @StoreID and IsEnabled = @IsEnabled and CreateDate < GETDATE() " +
                "GROUP BY Month( CreateDate ), Year(CreateDate) " +
                "ORDER BY Year(CreateDate) DESC, Month( CreateDate ) DESC; ",
                DataAccess.CreateParameterString( storeID ),
                DataAccess.CreateParameterBool( true ) );

            return table;
        }

        public IList<Blog> GetRecentPost( string storeID, string limitRow )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT TOP " + limitRow + " " + AllFields +
                "FROM Blog, BlogStore " +
                "WHERE Blog.BlogID = BlogStore.BlogID and StoreID = @StoreID and IsEnabled = @IsEnabled and CreateDate < GETDATE() " +
                "ORDER BY CreateDate DESC; ",
                DataAccess.CreateParameterString( storeID ),
                DataAccess.CreateParameterBool( true ) );

            return ConvertTableToList( table );

        }

        public IList<Blog> GetBlogListByDate(
            string month,
            string year,
            int startIndex,
            int endIndex,
            out int howManyItems,
            string storeID )
        {
            DataTable full = DataAccess.ExecuteSelect(
                "SELECT DISTINCT " + AllFields +
                "FROM Blog, BlogStore " +
                "WHERE Month(CreateDate) = @Month and YEAR(CreateDate) = @Year and StoreID = @StoreID and IsEnabled = @IsEnabled and CreateDate < GETDATE() " +
                "ORDER BY CreateDate DESC; ",
                DataAccess.CreateParameterString( month ),
                DataAccess.CreateParameterString( year ),
                DataAccess.CreateParameterString( storeID ),
                DataAccess.CreateParameterBool( true ) );

            howManyItems = full.Rows.Count;

            DataColumn[] keys = new DataColumn[] { full.Columns["BlogID"] };
            full.PrimaryKey = keys;
            return ConvertTableToList( full, startIndex, endIndex );
        }

        public IList<Blog> GetBlogListByCategoryID(
            string sortBy,
            string categoryID,
            int startIndex,
            int endIndex,
            out int howManyItems,
            string storeID,
            BoolFilter isEnabled)
        {
            DataTable full = _blogCategoryItemAccess.GetBlogsByCategoryID( sortBy, categoryID, storeID, isEnabled );

            howManyItems = full.Rows.Count;

            DataColumn[] keys = new DataColumn[] { full.Columns["BlogID"] };
            full.PrimaryKey = keys;
            return ConvertTableToList( full, startIndex, endIndex );
        }

        public int GetBlogCountByCategoryID( string categoryID, string storeID, BoolFilter isEnabled )
        {
            return _blogCategoryItemAccess.GetBlogCountByCategoryID( categoryID, storeID, isEnabled );
        }

        public IList<Blog> QuickSearch(
            string searchTerm,
            string[] searchFieldList,
            int startIndex,
            int endIndex,
            out int howManyItems,
            string storeID
            )
        {
            return ExecuteQuickSearch( searchTerm, searchFieldList, storeID, startIndex, endIndex, out howManyItems );
        }

        #endregion
    }
}
