﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain;
using Vevo.Base.Data.Caching;
using Vevo.Domain.Base;
using Vevo.Domain.Products;
using System.Web.UI;
using Vevo.Shared.Utilities;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class SpecificationGroupRepository : ISpecificationGroupRepository
    {

        private const string RepositoryName = "SpecificationGroupRepository";

        private const string GetOneSignature = "GetOne";
        private const string GetLocalesSignature = "GetLocales";
        private const string GetTableSchemaSignature = "GetTableSchema";
        private const string SearchSpecificationGroupSignature = "SearchSpecificationGroup";
        private const string GetAllSignature = "GetAll";

        private ICacheDependencyUtility _cacheDependency
            = new CacheDependencyUtility( new string[] { "SpecificationGroup", "SpecificationGroupLocale", "ProductSpecification" } );

        private DataCache _dataCache = new DataCache( RepositoryName );
        private SpecificationGroupRepositoryNoCache _repository = new SpecificationGroupRepositoryNoCache();

        #region Constructors

        public SpecificationGroupRepository()
        {
        }

        public SpecificationGroupRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region ISpecificationGroupRepository Members

        public IList<ILocale> GetLocales( string specificationGroupID )
        {
            IList<ILocale> list = (IList<ILocale>) _dataCache.GetValue( GetLocalesSignature, _cacheDependency, specificationGroupID );

            if (list == null)
            {
                list = _repository.GetLocales( specificationGroupID );
                _dataCache.SetValue( GetLocalesSignature, list, specificationGroupID );
            }

            return list;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue(
                   GetTableSchemaSignature, _cacheDependency
                   );

            if (list == null)
            {
                list = _repository.GetTableSchema();
                _dataCache.SetValue(
                    GetTableSchemaSignature,
                    list
                    );
            }

            return list;
        }

        public SpecificationGroup Save( SpecificationGroup specificationGroup )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            SpecificationGroup newSpecificationGroup = _repository.Save( specificationGroup );
            _dataCache.ClearRepositoryCache();
            return newSpecificationGroup;
        }

        public void Delete( string specificationGroupID )
        {
            _repository.Delete( specificationGroupID );
            _dataCache.ClearRepositoryCache();
        }

        public void DeleteLocalesByCultureID( string cultureID )
        {
            _repository.DeleteLocalesByCultureID( cultureID );
            _dataCache.ClearRepositoryCache();
        }

        public SpecificationGroup GetOne( Culture culture, string specificationGroupID )
        {
            SpecificationGroup result = (SpecificationGroup) _dataCache.GetValue(
                GetOneSignature, _cacheDependency, culture.CultureID, specificationGroupID );

            if (result == null)
            {
                result = _repository.GetOne( culture, specificationGroupID );
                _dataCache.SetValue( GetOneSignature, result, culture.CultureID, specificationGroupID );
            }

            return result;
        }

        public IList<SpecificationGroup> GetAll( Culture culture )
        {
            IList<SpecificationGroup> result = (IList<SpecificationGroup>) _dataCache.GetValue(
                 GetAllSignature, _cacheDependency,culture.CultureID
                 );

            if (result == null)
            {
                result = new List<SpecificationGroup>();
                result = _repository.GetAll( culture );
                _dataCache.SetValue(
                    GetAllSignature,
                    result,
                    culture.CultureID
                    );
            }
            return result;
        }

        public IList<SpecificationGroup> SearchSpecificationGroup( 
            Culture culture, 
            string sortBy, 
            SearchFilter searchFilter, 
            int startIndex, 
            int endIndex, 
            out int howManyItems )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchSpecificationGroupSignature,
                _cacheDependency,
                culture.CultureID,
                sortBy,
                searchFilter.ToString(),
                startIndex,
                endIndex
                );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchSpecificationGroup( culture, sortBy, searchFilter, startIndex, endIndex, out howManyItems );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchSpecificationGroupSignature,
                    result,
                    culture.CultureID,
                    sortBy,
                    searchFilter.ToString(),
                    startIndex,
                    endIndex );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<SpecificationGroup>) result.First;
        }

        #endregion
    }
}
