using System;
using System.Data;
using System.Collections.Generic;
using System.Configuration;

using Vevo.Domain.Configurations;
using Vevo.Domain.Orders;
using Vevo.Domain.Users;

namespace Vevo.Domain.Shipping
{
    /// <summary>
    /// Summary description for Shipping
    /// </summary>
    public abstract class ShippingMethod
    {
        #region Public Constances

        public const string NonRealTimeTrackingName = "Other";

        #endregion


        #region Private

        private ConfigurationCollection _configurations;

        private string _name;

        private ShippingOption _shippingOption;

        #endregion


        #region Protected

        protected IList<ICartItem> GetOverriddenShippingCartItems( CartItemGroup cartItemGroup )
        {
            IList<ICartItem> list = new List<ICartItem>();

            foreach (ICartItem cartItem in cartItemGroup.CartItems)
            {
                if (cartItem.Product.FixedShippingCost || cartItem.Product.FreeShippingCost)
                    list.Add( cartItem );
            }

            return list;
        }

        protected IList<ICartItem> GetNonOverriddenShippingCartItems( CartItemGroup cartItemGroup )
        {
            IList<ICartItem> list = new List<ICartItem>();

            foreach (ICartItem cartItem in cartItemGroup.CartItems)
            {
                if (!cartItem.Product.FixedShippingCost && !cartItem.Product.FreeShippingCost)
                    list.Add( cartItem );
            }

            return list;
        }

        protected decimal GetOverriddenShippingAmount( CartItemGroup cartItemGroup )
        {
            IList<ICartItem> list = GetOverriddenShippingCartItems( cartItemGroup );

            decimal amount = 0;
            foreach (ICartItem cartItem in list)
            {
                if ((!cartItem.IsPromotion) && (!cartItem.Product.FreeShippingCost))
                    amount += cartItem.Quantity
                        * cartItem.Product.GetOverriddenShippingCost( ShippingOption.ShippingID );
            }

            return amount;
        }

        #endregion


        #region Constructors

        public ShippingMethod(
            ShippingOption shippingOption )
        {
            _configurations = DataAccessContext.Configurations;
            _shippingOption = shippingOption;
        }

        #endregion


        #region Public Properties

        public static ShippingMethod Null
        {
            get { return NullShippingMethod.Instance; }
        }

        public virtual bool IsNull
        {
            get { return false; }
        }

        public ShippingOption ShippingOption
        {
            get
            {
                return _shippingOption;
            }
        }

        public virtual string Name
        {
            get
            {
                return _name;
            }
            set
            {
                if (value.LastIndexOf( "(" ) > 0)
                    _name = value.Substring( 0, value.LastIndexOf( "(" ) ).Trim();
                else
                    _name = value;
            }
        }

        public virtual string ShippingID
        {
            get
            {
                return ShippingOption.ShippingID;
            }
        }

        #endregion


        #region Public Methods

        public abstract decimal GetShippingCost(
            CartItemGroup cartItemGroup,
            WholesaleStatus wholesaleStatus,
            decimal discount );

        public virtual string GetOnlineTrackingName()
        {
            return NonRealTimeTrackingName;
        }

        public virtual decimal GetHandlingFee(
            CartItemGroup cartItemGroup,
            WholesaleStatus wholesaleStatus )
        {
            if (_configurations.GetBoolValue( "HandlingFeeEnabled" )
                && !IsFreeShippingForNonOverriddenItems( cartItemGroup, wholesaleStatus ))
                return ShippingOption.HandlingFee;
            else
                return 0;
        }

        public bool IsFreeShippingForNonOverriddenItems(
            CartItemGroup cartItemGroup,
            WholesaleStatus wholesaleStatus )
        {
            IList<ICartItem> list = GetNonOverriddenShippingCartItems( cartItemGroup );

            if (list.Count == 0)
            {
                return true;
            }
            else if (!ShippingOption.FreeShipping)
            {
                return false;
            }
            else
            {
                return (GetSumSubtotal( list, wholesaleStatus )
                    >= ShippingOption.FreeShippingValue);
            }
        }

        public decimal GetSumSubtotal( IList<ICartItem> cartItems, WholesaleStatus wholesaleStatus )
        {
            decimal sum = 0;

            foreach (ICartItem cartItem in cartItems)
            {
                sum += cartItem.GetSubtotal( wholesaleStatus );
            }

            return sum;
        }

        #endregion

    }
}
