﻿using System;
using System.Collections;
using System.Collections.Generic;
using eBay.Service.Call;
using eBay.Service.Core.Sdk;
using eBay.Service.Core.Soap;
using Vevo.Domain;
using Vevo.Deluxe.Domain.EBay;
using Vevo.Domain.Products;
using Vevo.Domain.Users;
using Vevo.Shared.Utilities;
using Vevo.Deluxe;
using Vevo.Base.Domain;
using Vevo.Shared.DataAccess;

namespace Vevo.eBay
{
    public class EBayAccess
    {
        #region private variable
        private string _storefrontURL = String.Empty;
        private Boolean _eBayHasWarning = false;
        private Boolean _eBayHasError = false;
        private IList<EBayErrorType> _eBayErrorTypeList;
        #endregion

        #region public property
        public Boolean HasWarning { get { return _eBayHasWarning; } }
        public Boolean HasError { get { return _eBayHasError; } }
        public IList<EBayErrorType> ErrorTypeList { get { return _eBayErrorTypeList; } }
        #endregion

        #region private methods
        private CurrencyCodeType GetCurrencyCodeType( EBayTemplate template )
        {
            switch (template.EBayListSite)
            {
                case "UK":
                    return CurrencyCodeType.GBP;
                case "Germany":
                    return CurrencyCodeType.EUR;
                default:
                    return CurrencyCodeType.USD;
            }
        }

        private int GetPounds( double weight )
        {
            return Convert.ToInt32( Math.Truncate( weight ) );
        }

        private double GetOunces( double weight )
        {
            int pounds = GetPounds( weight );
            double ounces = weight - pounds;
            return Math.Round( ounces * 16, 2 );
        }

        private double GetWeightInPounds( double weight )
        {
            double result = weight;

            if (DataAccessContext.Configurations.GetValue( "WeightUnit" ) == "KG")
            {
                return ConvertUtilities.ToDouble( WeightUtilities.ConvertKgToPounds( result ) );
            }
            else
            {
                return result;
            }
        }

        private ShippingTypeCodeType SetUpShippingType( EBayTemplate eBayTemplate )
        {
            if (eBayTemplate.DomesticShippingType == "Flat")
            {
                if (String.IsNullOrEmpty( eBayTemplate.InternationalShippingType ) || eBayTemplate.InternationalShippingType == "Flat")
                {
                    return ShippingTypeCodeType.Flat;
                }
                else
                {
                    return ShippingTypeCodeType.FlatDomesticCalculatedInternational;
                }
            }
            else
            {
                if (eBayTemplate.InternationalShippingType == "Flat")
                {
                    return ShippingTypeCodeType.CalculatedDomesticFlatInternational;
                }
                else
                {
                    return ShippingTypeCodeType.Calculated;
                }
            }
        }

        private ShippingServiceOptionsType SetUpDomesticLocalPickUp( EBayTemplate template, int priority )
        {
            ShippingServiceOptionsType shippingOptionType = new ShippingServiceOptionsType();
            shippingOptionType.ShippingServicePriority = priority;
            shippingOptionType.ShippingService = ShippingServiceCodeType.Pickup.ToString();
            shippingOptionType.ShippingServiceCost = ConvertDoubleToAmoutType( template, 0 );
            return shippingOptionType;
        }

        private double ConvertToTemplateCurrency( EBayTemplate template, double price )
        {
            Currency currency = DataAccessContext.CurrencyRepository.GetOne( GetCurrencyCodeType( template ).ToString() );
            return price * currency.ConversionRate;
        }

        private AmountType ConvertDoubleToAmoutType( EBayTemplate template, double price )
        {
            Currency currency = DataAccessContext.CurrencyRepository.GetOne( GetCurrencyCodeType( template ).ToString() );
            AmountType amount = new AmountType();
            amount.currencyID = GetCurrencyCodeType( template );
            amount.Value = price * currency.ConversionRate;
            return amount;
        }

        private CalculatedShippingRateType GetShippingRateType( EBayTemplate eBayTemplate, Product product )
        {
            double productWeight = GetWeightInPounds( product.Weight );

            CalculatedShippingRateType calculatedShipping = new CalculatedShippingRateType();

            if (eBayTemplate.DomesticShippingType != "Flat")
            {
                calculatedShipping.ShippingPackage =
                        (ShippingPackageCodeType) Enum.Parse( typeof( ShippingPackageCodeType ),
                        eBayTemplate.DomesticShippingPackage, true );

                calculatedShipping.PackagingHandlingCosts = ConvertDoubleToAmoutType( eBayTemplate, eBayTemplate.DomesticShippingHandlingCost );
            }

            if (eBayTemplate.IsInternationalShippingEnabled && eBayTemplate.InternationalShippingType != "Flat")
            {
                if (eBayTemplate.DomesticShippingType == "Flat")
                {
                    calculatedShipping.ShippingPackage = (ShippingPackageCodeType) Enum.Parse
                        ( typeof( ShippingPackageCodeType ), eBayTemplate.InternationalShippingPackage, true );
                }


                if (eBayTemplate.IsInternationalShippingEnabled)
                {
                    calculatedShipping.InternationalPackagingHandlingCosts =
                        ConvertDoubleToAmoutType( eBayTemplate, eBayTemplate.InternationalShippingHandlingCost );
                }
            }

            MeasureType majorWeightType = new MeasureType();
            majorWeightType.measurementSystemSpecified = true;
            majorWeightType.Value = (decimal) GetPounds( productWeight );

            MeasureType minorWeightType = new MeasureType();
            minorWeightType.measurementSystemSpecified = true;
            minorWeightType.Value = (decimal) GetOunces( productWeight );

            calculatedShipping.MeasurementUnit = MeasurementSystemCodeType.English;
            calculatedShipping.WeightMajor = majorWeightType;
            calculatedShipping.WeightMinor = minorWeightType;

            return calculatedShipping;
        }

        private InternationalShippingServiceOptionsTypeCollection SetUpInternationalShipping(
            EBayTemplate eBayTemplate,
            Product product )
        {
            InternationalShippingServiceOptionsTypeCollection interShippingCollection = new InternationalShippingServiceOptionsTypeCollection();

            InternationalShippingServiceOptionsType interOption1 = new InternationalShippingServiceOptionsType();
            interOption1.ShippingServicePriority = 1;

            interOption1.ShippingService =
                ((ShippingServiceCodeType) Enum.Parse( typeof( ShippingServiceCodeType ),
                eBayTemplate.InternationalShippingMethod1, true )).ToString();

            if (eBayTemplate.InternationalShippingType == "Flat")
            {
                interOption1.ShippingServiceCost = ConvertDoubleToAmoutType( eBayTemplate, eBayTemplate.InternationalShippingFirstItem1 );
                interOption1.ShippingServiceAdditionalCost = ConvertDoubleToAmoutType( eBayTemplate, eBayTemplate.InternationalShippingNextItem1 );
            }

            StringCollection shipTo1 = new StringCollection();
            shipTo1.Add( eBayTemplate.InternationalShippingShipTo1 );
            interOption1.ShipToLocation = shipTo1;

            interShippingCollection.Add( interOption1 );

            if (!eBayTemplate.InternationalShippingShipTo2.Equals( "None" ))
            {
                InternationalShippingServiceOptionsType interOption2 = new InternationalShippingServiceOptionsType();
                interOption2.ShippingServicePriority = 2;
                interOption2.ShippingService =
                    ((ShippingServiceCodeType) Enum.Parse( typeof( ShippingServiceCodeType ), eBayTemplate.InternationalShippingMethod2, true )).ToString();

                if (eBayTemplate.InternationalShippingType == "Flat")
                {
                    interOption2.ShippingServiceCost = ConvertDoubleToAmoutType( eBayTemplate, eBayTemplate.InternationalShippingFirstItem2 );
                    interOption2.ShippingServiceAdditionalCost = ConvertDoubleToAmoutType( eBayTemplate, eBayTemplate.InternationalShippingNextItem2 );
                }

                StringCollection shipTo2 = new StringCollection();
                shipTo2.Add( eBayTemplate.InternationalShippingShipTo2 );
                interOption2.ShipToLocation = shipTo2;

                interShippingCollection.Add( interOption2 );
            }

            if (!eBayTemplate.InternationalShippingShipTo3.Equals( "None" ))
            {
                InternationalShippingServiceOptionsType interOption3 = new InternationalShippingServiceOptionsType();
                interOption3.ShippingServicePriority = 3;
                interOption3.ShippingService =
                    ((ShippingServiceCodeType) Enum.Parse( typeof( ShippingServiceCodeType ), eBayTemplate.InternationalShippingMethod3, true )).ToString();

                if (eBayTemplate.InternationalShippingType == "Flat")
                {
                    interOption3.ShippingServiceCost = ConvertDoubleToAmoutType( eBayTemplate, eBayTemplate.InternationalShippingFirstItem3 );
                    interOption3.ShippingServiceAdditionalCost = ConvertDoubleToAmoutType( eBayTemplate, eBayTemplate.InternationalShippingNextItem3 );
                }

                StringCollection shipTo3 = new StringCollection();
                shipTo3.Add( eBayTemplate.InternationalShippingShipTo3 );
                interOption3.ShipToLocation = shipTo3;

                interShippingCollection.Add( interOption3 );
            }

            return interShippingCollection;
        }

        private ShippingServiceOptionsTypeCollection SetUpDomesticShipping( EBayTemplate eBayTemplate, Product product )
        {

            ShippingServiceOptionsTypeCollection serviceOptionCollection = new ShippingServiceOptionsTypeCollection();

            if (eBayTemplate.DomesticShippingMethod1.Equals( "LocalDelivery" ))
            {
                ShippingServiceOptionsType domesticOption1 = SetUpDomesticLocalPickUp( eBayTemplate, 1 );
                serviceOptionCollection.Add( domesticOption1 );
            }
            else
            {
                ShippingServiceOptionsType domesticOption1 = new ShippingServiceOptionsType();
                domesticOption1.ShippingServicePriority = 1;

                domesticOption1.ShippingService =
                    ((ShippingServiceCodeType) Enum.Parse( typeof( ShippingServiceCodeType ), eBayTemplate.DomesticShippingMethod1, true )).ToString();

                if (eBayTemplate.DomesticShippingType == "Flat")
                {
                    domesticOption1.ShippingServiceCost = ConvertDoubleToAmoutType( eBayTemplate, eBayTemplate.DomesticShippingFirstItem1 );
                    domesticOption1.ShippingServiceAdditionalCost = ConvertDoubleToAmoutType( eBayTemplate, eBayTemplate.DomesticShippingNextItem1 );
                }

                if (eBayTemplate.IsFreeDomesticShippingCost)
                    domesticOption1.FreeShipping = true;

                serviceOptionCollection.Add( domesticOption1 );
            }

            if (!eBayTemplate.DomesticShippingMethod2.Equals( "None" ))
            {
                if (eBayTemplate.DomesticShippingMethod2.Equals( "LocalDelivery" ))
                {
                    if (!eBayTemplate.DomesticShippingMethod1.Equals( "LocalDelivery" )
                        && !eBayTemplate.DomesticShippingMethod3.Equals( "LocalDelivery" ))
                    {
                        ShippingServiceOptionsType domesticOption2 = SetUpDomesticLocalPickUp( eBayTemplate, 2 );
                        serviceOptionCollection.Add( domesticOption2 );
                    }
                }
                else
                {
                    ShippingServiceOptionsType domesticOption2 = new ShippingServiceOptionsType();
                    domesticOption2.ShippingServicePriority = 2;

                    domesticOption2.ShippingService =
                        ((ShippingServiceCodeType) Enum.Parse( typeof( ShippingServiceCodeType ), eBayTemplate.DomesticShippingMethod2, true )).ToString();

                    if (eBayTemplate.DomesticShippingType == "Flat")
                    {
                        domesticOption2.ShippingServiceCost = ConvertDoubleToAmoutType( eBayTemplate, eBayTemplate.DomesticShippingFirstItem2 );
                        domesticOption2.ShippingServiceAdditionalCost = ConvertDoubleToAmoutType( eBayTemplate, eBayTemplate.DomesticShippingNextItem2 );
                    }

                    serviceOptionCollection.Add( domesticOption2 );
                }
            }

            if (!eBayTemplate.DomesticShippingMethod3.Equals( "None" ))
            {
                if (eBayTemplate.DomesticShippingMethod3.Equals( "LocalDelivery" ))
                {
                    if (!eBayTemplate.DomesticShippingMethod1.Equals( "LocalDelivery" )
                        && !eBayTemplate.DomesticShippingMethod2.Equals( "LocalDelivery" ))
                    {
                        ShippingServiceOptionsType domesticOption3 = SetUpDomesticLocalPickUp( eBayTemplate, 3 );
                        serviceOptionCollection.Add( domesticOption3 );
                    }
                }
                else
                {
                    ShippingServiceOptionsType domesticOption3 = new ShippingServiceOptionsType();
                    domesticOption3.ShippingServicePriority = 3;

                    domesticOption3.ShippingService =
                        ((ShippingServiceCodeType) Enum.Parse( typeof( ShippingServiceCodeType ), eBayTemplate.DomesticShippingMethod3, true )).ToString();

                    if (eBayTemplate.DomesticShippingType == "Flat")
                    {
                        domesticOption3.ShippingServiceCost = ConvertDoubleToAmoutType( eBayTemplate, eBayTemplate.DomesticShippingFirstItem3 );
                        domesticOption3.ShippingServiceAdditionalCost = ConvertDoubleToAmoutType( eBayTemplate, eBayTemplate.DomesticShippingNextItem3 );
                    }

                    serviceOptionCollection.Add( domesticOption3 );
                }
            }

            return serviceOptionCollection;
        }

        private ShippingDetailsType BuildShippingDetails( EBayTemplate template, Product product )
        {
            double productWeight = GetWeightInPounds( product.Weight );

            ShippingDetailsType shippingDetail = new ShippingDetailsType();
            shippingDetail.GetItFast = template.DomesticShippingShowGetItFast;
            shippingDetail.PaymentInstructions = template.CheckoutInstruction;
            shippingDetail.ShippingType = SetUpShippingType( template );

            shippingDetail.ShippingServiceOptions = SetUpDomesticShipping( template, product );

            if (template.IsInternationalShippingEnabled)
            {
                shippingDetail.InternationalShippingServiceOption = SetUpInternationalShipping( template, product );
            }

            if (template.DomesticShippingType != "Flat" ||

                (!String.IsNullOrEmpty( template.InternationalShippingType ) && template.InternationalShippingType != "Flat"))
            {
                shippingDetail.CalculatedShippingRate = GetShippingRateType( template, product );
            }

            if (template.SalesTaxType == "Charge")
            {
                float salesTaxPercent = float.Parse( template.SalesTaxValue.ToString() );
                SalesTaxType salesTax = new SalesTaxType();
                salesTax.SalesTaxPercent = salesTaxPercent;
                salesTax.SalesTaxState = template.SalesTaxState;
                salesTax.ShippingIncludedInTax = template.IsTaxableShippingCost;

                shippingDetail.SalesTax = salesTax;
            }

            return shippingDetail;
        }

        private ItemType ConvertProductToItemType(
            Product product,
            EBayTemplate template,
            EBayCategory category,
            Boolean isScheduleDate,
            DateTime schedultDateTime,
            Boolean isVerifyAddItem )
        {
            ItemType item = new ItemType();
            item.Title = product.Name;
            item.Description = product.ShortDescription;
            item.Currency = GetCurrencyCodeType( template );

            double productPrice = 0.0;
            foreach (ProductPrice price in product.ProductPrices)
            {
                if (price.StoreID.Equals( "0" ))    //use defalut price
                {
                    productPrice = (double) price.Price;
                    break;
                }
            }

            if (template.SellingMethod.Equals( "Online Auction" ))
            {
                item.ListingType = ListingTypeCodeType.Chinese; //chinese = auction

                if (template.UseBuyItNowPrice)
                {
                    item.BuyItNowPrice = new AmountType();
                    item.BuyItNowPrice.currencyID = GetCurrencyCodeType( template );
                    double templateBuyItNowPrice = ConvertUtilities.ToDouble( template.BuyItNowPriceValue );

                    switch (template.BuyItNowPriceType)
                    {
                        case "ProductPrice":
                            item.BuyItNowPrice.Value = ConvertToTemplateCurrency( template, productPrice );
                            break;
                        case "PricePlusAmount":
                            item.BuyItNowPrice.Value = ConvertToTemplateCurrency( template, productPrice + templateBuyItNowPrice );
                            break;
                        case "PricePlusPercentage":
                            item.BuyItNowPrice.Value = ConvertToTemplateCurrency( template, (productPrice + (productPrice * (templateBuyItNowPrice / 100))) );
                            break;
                        case "CustomPrice":
                            item.BuyItNowPrice.Value = ConvertToTemplateCurrency( template, templateBuyItNowPrice );
                            break;
                        default:
                            item.BuyItNowPrice.Value = ConvertToTemplateCurrency( template, productPrice );
                            break;
                    }

                }
                if (template.UseReservePrice)
                {
                    item.ReservePrice = new AmountType();
                    item.ReservePrice.currencyID = GetCurrencyCodeType( template );
                    double templateReservePrice = ConvertUtilities.ToDouble( template.ReservePriceValue );

                    switch (template.ReservePriceType)
                    {
                        case "ProductPrice":
                            item.ReservePrice.Value = ConvertToTemplateCurrency( template, productPrice );
                            break;
                        case "PricePlusAmount":
                            item.ReservePrice.Value = ConvertToTemplateCurrency( template, productPrice + templateReservePrice );
                            break;
                        case "PricePlusPercentage":
                            item.ReservePrice.Value = ConvertToTemplateCurrency( template, (productPrice + (productPrice * (templateReservePrice / 100))) );
                            break;
                        case "CustomPrice":
                            item.ReservePrice.Value = ConvertToTemplateCurrency( template, templateReservePrice );
                            break;
                        default:
                            item.ReservePrice.Value = ConvertToTemplateCurrency( template, productPrice );
                            break;
                    }
                }

                item.StartPrice = new AmountType();
                item.StartPrice.currencyID = GetCurrencyCodeType( template );
                double templateStartPrice = ConvertUtilities.ToDouble( template.StartingPriceValue );

                switch (template.StartingPriceType)
                {
                    case "ProductPrice":
                        item.StartPrice.Value = ConvertToTemplateCurrency( template, productPrice );
                        break;
                    case "PricePlusAmount":
                        item.StartPrice.Value = ConvertToTemplateCurrency( template, productPrice + templateStartPrice );
                        break;
                    case "PricePlusPercentage":
                        item.StartPrice.Value = ConvertToTemplateCurrency( template, (productPrice + (productPrice * (templateStartPrice / 100))) );
                        break;
                    case "CustomPrice":
                        item.StartPrice.Value = ConvertToTemplateCurrency( template, templateStartPrice );
                        break;
                    default:
                        item.StartPrice.Value = ConvertToTemplateCurrency( template, productPrice );
                        break;
                }

                item.Quantity = 1;  //The Quantity value for auction-style listings must always be 1
            }
            else
            {
                item.ListingType = ListingTypeCodeType.FixedPriceItem;
                item.StartPrice = new AmountType();
                item.StartPrice.currencyID = GetCurrencyCodeType( template );
                double templateStartPrice = ConvertUtilities.ToDouble( template.BuyItNowPriceValue );

                switch (template.BuyItNowPriceType)
                {
                    case "ProductPrice":
                        item.StartPrice.Value = ConvertToTemplateCurrency( template, productPrice );
                        break;
                    case "PricePlusAmount":
                        item.StartPrice.Value = ConvertToTemplateCurrency( template, productPrice + templateStartPrice );
                        break;
                    case "PricePlusPercentage":
                        item.StartPrice.Value = ConvertToTemplateCurrency( template, (productPrice + (productPrice * (templateStartPrice / 100))) );
                        break;
                    case "CustomPrice":
                        item.StartPrice.Value = ConvertToTemplateCurrency( template, template.BuyItNowPriceValue );
                        break;
                    default:
                        item.StartPrice.Value = ConvertToTemplateCurrency( template, productPrice );
                        break;
                }

                item.Quantity = template.Quantity;
            }

            item.ListingDuration = "Days_" + template.ListingDuration;

            if (isScheduleDate)
            {
                item.ScheduleTimeSpecified = true;
                item.ScheduleTime = schedultDateTime;
            }

            item.PrivateListing = template.IsPrivateListing;

            item.Location = template.ProductState;
            item.Country = (CountryCodeType) Enum.Parse( typeof( CountryCodeType ), template.ProductCountry, true );
            item.PostalCode = template.ProductZip;

            CategoryType primaryCategory = new CategoryType();
            primaryCategory.CategoryID = category.PimaryCategoryID;
            item.PrimaryCategory = primaryCategory;

            if (!String.IsNullOrEmpty( category.SecondaryCategoryID ))
            {
                CategoryType secondaryCategory = new CategoryType();
                secondaryCategory.CategoryID = category.SecondaryCategoryID;
                item.SecondaryCategory = secondaryCategory;
            }

            BuyerPaymentMethodCodeTypeCollection paymentCollection = new BuyerPaymentMethodCodeTypeCollection();

            string[] selectedPaymentList = template.PaymentMethod.Split( ',' );
            foreach (string selectedPayment in selectedPaymentList)
            {
                paymentCollection.Add( (BuyerPaymentMethodCodeType) Enum.Parse( typeof( BuyerPaymentMethodCodeType ), selectedPayment, true ) );
            }
            item.PaymentMethods = new BuyerPaymentMethodCodeTypeCollection();
            item.PaymentMethods.AddRange( paymentCollection );
            item.PayPalEmailAddress = template.PayPalEmailAddress;

            item.ConditionID = ConvertUtilities.ToInt32( template.ProductConditionID );
            item.ConditionDisplayName = template.ProductConditionName;

            if (template.IsShowImage)
            {
                if (product.ProductImages.Count > 0)
                {
                    string newStoreFrontURL = _storefrontURL;
                    if (_storefrontURL.Contains( "https" ))
                        newStoreFrontURL = _storefrontURL.Replace( "https", "http" );
                    string imagePath = newStoreFrontURL + product.ProductImages[0].RegularImage;
                    PictureDetailsType pictureDetailType = new PictureDetailsType();

                    if (DataAccessContext.Configurations.GetValue( "eBayConfigListingMode" ).ToString().Equals( "https://api.ebay.com/wsapi" ))
                    {
                        /*
                         * Supported in Production only (not Sandbox).
                         * The URL for a picture used as the Gallery thumbnail.
                         * On a free-Gallery site, if there is a PictureURL, the PictureURL fills in or replaces the GalleryURL.
                         * Note that GalleryURL is ignored if GalleryType is None or unspecified. 
                         * If the value of GalleryType is Gallery or Featured, 
                         * you can either specify GalleryURL or allow eBay to use another picture that you have included in the listing. 
                         */
                        if (!template.GalleryType.Equals( "None" ))
                            pictureDetailType.GalleryURL = imagePath;
                    }

                    if (template.GalleryType.Equals( "Featured" ))
                    {
                        pictureDetailType.GalleryType = GalleryTypeCodeType.Featured;
                        pictureDetailType.GalleryDuration = template.GalleryDuration;
                    }
                    else
                    {
                        pictureDetailType.GalleryType = (GalleryTypeCodeType) Enum.Parse( typeof( GalleryTypeCodeType ), template.GalleryType, true );
                    }

                    if (isVerifyAddItem)
                    {
                        //imagePath = "http://i2.ebayimg.com/abc/M28/dummy.jpg";
                    }

                    imagePath = imagePath.Replace( " ", "%20" );
                    StringCollection externalImageURLCollection = new StringCollection();
                    externalImageURLCollection.Add( imagePath );
                    pictureDetailType.ExternalPictureURL = externalImageURLCollection;
                    pictureDetailType.PhotoDisplay = PhotoDisplayCodeType.None;

                    item.PictureDetails = pictureDetailType;
                }
            }

            if (template.DomesticShippingShowGetItFast)
            {
                item.GetItFast = true;
                item.DispatchTimeMax = 1;
            }
            else
            {
                item.DispatchTimeMax = ConvertUtilities.ToInt32( template.HandlingTime );
            }

            // shipping details
            item.ShippingDetails = BuildShippingDetails( template, product );

            item.ReturnPolicy = new ReturnPolicyType();
            if (template.IsAcceptReturn)
            {
                item.ReturnPolicy.ReturnsAcceptedOption = "ReturnsAccepted";
                item.ReturnPolicy.RefundOption = template.ReturnOffered;
                item.ReturnPolicy.ReturnsWithinOption = "Days_" + template.ReturnPeriod;
                item.ReturnPolicy.ShippingCostPaidByOption = template.ReturnShippingPaidBy;
                item.ReturnPolicy.Description = template.ReturnPolicyAdditionInfo;
            }
            else
            {
                item.ReturnPolicy.ReturnsAcceptedOption = "ReturnsNotAccepted";
            }

            //additional option
            if (!template.CounterStyle.Equals( "None" ))
            {
                item.HitCounter = (HitCounterCodeType) Enum.Parse( typeof( HitCounterCodeType ), template.CounterStyle, true );
            }

            if (template.IsBoldListingTitle)
            {
                ListingEnhancementsCodeTypeCollection enchantCollection = new ListingEnhancementsCodeTypeCollection();
                enchantCollection.Add( ListingEnhancementsCodeType.BoldTitle );
                item.ListingEnhancement = enchantCollection;
            }

            return item;
        }

        private IList<EBayFee> ConvertFeeCollectionToList( FeeTypeCollection feeCollection )
        {
            IList<EBayFee> feeList = new List<EBayFee>();
            foreach (FeeType eBayType in feeCollection)
            {
                EBayFee objFee = new EBayFee();
                objFee.Name = eBayType.Name;
                objFee.Value = eBayType.Fee.Value;
                objFee.Currency = eBayType.Fee.currencyID.ToString();
                feeList.Add( objFee );
            }

            return feeList;
        }

        private IList<EBayCategory> ConvertCategoryCollectionToList( CategoryTypeCollection catCollection )
        {
            IList<EBayCategory> catList = new List<EBayCategory>();
            foreach (CategoryType catType in catCollection)
            {
                EBayCategory categoryItem = new EBayCategory();
                categoryItem.PimaryCategoryID = catType.CategoryID;
                categoryItem.CategoryLevel = catType.CategoryLevel;
                categoryItem.PimaryCategoryName = catType.CategoryName;

                string[] parentIDList = new string[catType.CategoryParentID.Count];
                for (int i = 0; i < parentIDList.Length; i++)
                {
                    parentIDList[i] = catType.CategoryParentID[i];
                }

                categoryItem.CategoryParentID = parentIDList;
                categoryItem.IsExpire = catType.Expired;
                categoryItem.IsLeafCategory = catType.LeafCategory;
                categoryItem.IsLSD = catType.LSD;
                categoryItem.IsVirtual = catType.Virtual;

                catList.Add( categoryItem );
            }

            return catList;
        }

        private IList<EBayCategoryFeature> ConvertCategoryFeatureCollectionToList( CategoryFeatureTypeCollection featureCollection )
        {
            IList<EBayCategoryFeature> catFeatureList = new List<EBayCategoryFeature>();
            foreach (CategoryFeatureType featureType in featureCollection)
            {
                EBayCategoryFeature catFeature = new EBayCategoryFeature();
                catFeature.IsPayPalRequired = featureType.PayPalRequired;
                catFeature.IsProductVariationEnabled = featureType.VariationsEnabled;
                catFeature.IsReturnPolicyEnabled = featureType.ReturnPolicyEnabled;
                catFeature.MinimumReservePrice = featureType.MinimumReservePrice;

                if (featureType.ConditionEnabled != ConditionEnabledCodeType.Disabled)
                {
                    catFeature.IsProductConditionEnabled = true;
                    ConditionValuesType conditionType = featureType.ConditionValues;
                    ConditionTypeCollection conditionCollection = conditionType.Condition;

                    IList<EBayProductCondition> productConditionList = new List<EBayProductCondition>();
                    foreach (ConditionType condition in conditionCollection)
                    {
                        EBayProductCondition productCondition = new EBayProductCondition();
                        productCondition.ConditionID = condition.ID.ToString();
                        productCondition.ConditionName = condition.DisplayName;
                        productConditionList.Add( productCondition );
                    }
                    catFeature.ProductCondition = productConditionList;
                }

                catFeatureList.Add( catFeature );
            }

            return catFeatureList;
        }

        private EBayItem SetEBayItem( ItemType eBayItemType )
        {
            EBayItem item = new EBayItem();
            item.ID = eBayItemType.ItemID;
            item.Title = eBayItemType.Title;
            item.PrimaryCategoryID = eBayItemType.PrimaryCategory.CategoryID;

            if (eBayItemType.SecondaryCategory != null)
                item.SecondaryCategoryID = eBayItemType.SecondaryCategory.CategoryID;

            item.ViewUrl = eBayItemType.ListingDetails.ViewItemURL;
            item.QuantityLeft = eBayItemType.Quantity;
            item.Type = eBayItemType.ListingType.ToString();
            if (item.Type.Equals( "Chinese" ))
            {
                item.Bidding_Count = eBayItemType.SellingStatus.BidCount;
                item.Bidding_CurrentPrice = eBayItemType.SellingStatus.CurrentPrice.Value;
                if (eBayItemType.ReservePrice != null)
                    item.Bidding_ReservePrice = eBayItemType.ReservePrice.Value;
                if (eBayItemType.BuyItNowPrice != null)
                    item.Bidding_BuyItNowPrice = eBayItemType.BuyItNowPrice.Value;
                item.Bidding_StartingPrice = eBayItemType.StartPrice.Value;
            }
            item.ListingStatus = eBayItemType.SellingStatus.ListingStatus.ToString();
            item.BuyItNowPrice = eBayItemType.StartPrice.Value;
            item.StartTime = eBayItemType.ListingDetails.StartTime;
            item.IsBuyItNowAvailable = eBayItemType.ListingDetails.BuyItNowAvailable;
            item.LotSize = eBayItemType.LotSize;
            item.Currency = eBayItemType.StartPrice.currencyID.ToString();

            return item;
        }
        #endregion

        #region public methods
        public ApiContext GetApiContext()
        {
            ApiContext apiContext = new ApiContext();
            apiContext.SoapApiServerUrl = DataAccessContext.Configurations.GetValue( "eBayConfigListingMode" ).ToString();
            ApiCredential apiCredential = new ApiCredential();
            apiCredential.eBayToken = DataAccessContext.Configurations.GetValue( "eBayConfigToken" ).ToString();
            apiContext.ApiCredential = apiCredential;
            return apiContext;

        }

        public ApiContext GetApiContext( string listSite )
        {
            ApiContext apiContext = new ApiContext();
            apiContext.SoapApiServerUrl = DataAccessContext.Configurations.GetValue( "eBayConfigListingMode" ).ToString();
            ApiCredential apiCredential = new ApiCredential();
            apiCredential.eBayToken = DataAccessContext.Configurations.GetValue( "eBayConfigToken" ).ToString();
            apiContext.ApiCredential = apiCredential;
            apiContext.Site = (SiteCodeType) Enum.Parse( typeof( SiteCodeType ), listSite, true );

            return apiContext;

        }
        public EBayAccess()
        {
        }

        public EBayAccess( string storefrontURL )
        {
            _storefrontURL = storefrontURL;
        }

        public string GetOfficialTime( EBayTemplate template, out string ack )
        {
            if (!KeyUtilities.IsDeluxeLicense( DataAccessHelper.DomainRegistrationkey, DataAccessHelper.DomainName ))
            {
                ack = string.Empty;
                return string.Empty;
            }
            ApiContext apiContext = GetApiContext( template.EBayListSite );
            GeteBayOfficialTimeCall timeCall = new GeteBayOfficialTimeCall( apiContext );

            timeCall.GeteBayOfficialTime();

            GeteBayOfficialTimeResponseType responseType = timeCall.ApiResponse;
            ack = responseType.Ack.ToString();
            return responseType.Timestamp.ToString();

        }

        public IList<EBayFee> VerifyAddItem(
            Product product,
            EBayTemplate template,
            EBayCategory category,
            Boolean isScheduleDate,
            DateTime scheduleDateTime )
        {
            if (!KeyUtilities.IsDeluxeLicense( DataAccessHelper.DomainRegistrationkey, DataAccessHelper.DomainName ))
            {
                return new List<EBayFee>();
            }

            ApiContext apiContext = GetApiContext( template.EBayListSite );
            ItemType item = ConvertProductToItemType( product, template, category, isScheduleDate, scheduleDateTime, true );

            VerifyAddItemCall verifyAddItemCall = new VerifyAddItemCall( apiContext );
            FeeTypeCollection feeCollection = new FeeTypeCollection();
            try
            {
                feeCollection = verifyAddItemCall.VerifyAddItem( item );
            }
            catch
            {
                //SaveLogFile.SaveLog(verifyAddItemCall.
                _eBayHasError = verifyAddItemCall.HasError;
                _eBayHasWarning = verifyAddItemCall.HasWarning;
                ApiException apiExcp = verifyAddItemCall.ApiException;
                _eBayErrorTypeList = new List<EBayErrorType>();
                foreach (ErrorType errorType in apiExcp.Errors)
                {
                    EBayErrorType error = new EBayErrorType();
                    error.ErrorCode = errorType.ErrorCode;
                    error.ErrorLongMessage = errorType.LongMessage;
                    error.ErrorShortMessage = errorType.ShortMessage;
                    _eBayErrorTypeList.Add( error );
                }
                return new List<EBayFee>();
            }

            return ConvertFeeCollectionToList( feeCollection );
        }

        public string AddItem(
            Product product,
            EBayTemplate template,
            EBayCategory category,
            Boolean isScheduleDate,
            DateTime scheduleDateTime )
        {
            if (!KeyUtilities.IsDeluxeLicense( DataAccessHelper.DomainRegistrationkey, DataAccessHelper.DomainName ))
            {
                return string.Empty;
            }

            ApiContext apiContext = GetApiContext( template.EBayListSite );
            ItemType item = ConvertProductToItemType( product, template, category, isScheduleDate, scheduleDateTime, false );

            AddItemCall addItemCall = new AddItemCall( apiContext );

            try
            {
                addItemCall.AddItem( item );
            }
            catch
            {
                Exception e = new Exception( addItemCall.AbstractResponse.Message );
                SaveLogFile.SaveLog( e );

                _eBayHasError = addItemCall.HasError;
                _eBayHasWarning = addItemCall.HasWarning;
                ApiException apiExcp = addItemCall.ApiException;
                _eBayErrorTypeList = new List<EBayErrorType>();
                foreach (ErrorType errorType in apiExcp.Errors)
                {
                    EBayErrorType error = new EBayErrorType();
                    error.ErrorCode = errorType.ErrorCode;
                    error.ErrorLongMessage = errorType.LongMessage;
                    error.ErrorShortMessage = errorType.ShortMessage;
                    _eBayErrorTypeList.Add( error );
                }
                return "";
            }

            return item.ItemID;
        }

        public bool EndListing( string itemNumber, EBayList.EBayListEndListingReason type )
        {
            if (!KeyUtilities.IsDeluxeLicense( DataAccessHelper.DomainRegistrationkey, DataAccessHelper.DomainName ))
            {
                return false;
            }

            ApiContext apiContext = GetApiContext();
            EndItemCall endCall = new EndItemCall( apiContext );

            try
            {
                string endType = type.ToString();
                EndReasonCodeType endListingRype = (EndReasonCodeType) Enum.Parse( typeof( EndReasonCodeType ), endType, true );
                endCall.EndItem( itemNumber, endListingRype );
            }
            catch
            {
                Exception e = new Exception( endCall.AbstractResponse.Message );
                SaveLogFile.SaveLog( e );
                return false;
            }
            return true;
        }

        public EBayItem GetItemDetail( string itemID, ApiContext apiContext )
        {
            if (!KeyUtilities.IsDeluxeLicense( DataAccessHelper.DomainRegistrationkey, DataAccessHelper.DomainName ))
            {
                return EBayItem.Null;
            }

            GetItemCall getItemCall = new GetItemCall( apiContext );

            ItemType item = new ItemType();
            EBayItem eBayItem = EBayItem.Null;

            try
            {
                item = getItemCall.GetItem( itemID );
            }
            catch
            {
                _eBayHasError = getItemCall.HasError;
                _eBayHasWarning = getItemCall.HasWarning;
                ApiException apiExcp = getItemCall.ApiException;
                _eBayErrorTypeList = new List<EBayErrorType>();
                foreach (ErrorType errorType in apiExcp.Errors)
                {
                    EBayErrorType error = new EBayErrorType();
                    error.ErrorCode = errorType.ErrorCode;
                    error.ErrorLongMessage = errorType.LongMessage;
                    error.ErrorShortMessage = errorType.ShortMessage;
                    _eBayErrorTypeList.Add( error );
                }
                return EBayItem.Null;
            }

            return SetEBayItem( item );
        }

        public EBayList GetListUpdateFromItem( EBayList inList )
        {
            if (!KeyUtilities.IsDeluxeLicense( DataAccessHelper.DomainRegistrationkey, DataAccessHelper.DomainName ))
            {
                return EBayList.Null;
            }

            EBayItem item = GetItemDetail( inList.ItemNumber, new EBayAccess().GetApiContext() );
            EBayList list = (EBayList) inList.Clone();

            list.ItemName = item.Title;
            list.ItemNumber = item.ID;
            list.LastStatus = (EBayList.EBayListStatus) Enum.Parse( typeof( EBayList.EBayListStatus ), item.ListingStatus, true );
            list.LastUpdate = DateTime.Now;
            list.ListDate = item.StartTime;
            list.ListType = (EBayList.EBayListType) Enum.Parse( typeof( EBayList.EBayListType ), item.Type, true );
            list.QtyLeft = item.QuantityLeft;
            list.BidAmount = item.Bidding_Count;
            list.BidPrice = (decimal) item.Bidding_CurrentPrice;
            if (list.ListType.ToString().Equals( "Chinese" ))
            {
                list.BuyItNowPrice = (decimal) item.Bidding_BuyItNowPrice;
            }
            else
            {
                list.BuyItNowPrice = (decimal) item.BuyItNowPrice;
            }

            return list;
        }

        public IList<EBayOrder> GetOrders( DateTime start, DateTime end )
        {
            if (!KeyUtilities.IsDeluxeLicense( DataAccessHelper.DomainRegistrationkey, DataAccessHelper.DomainName ))
            {
                return new List<EBayOrder>();
            }

            ApiContext apiContext = GetApiContext();
            GetOrdersCall apicall = new GetOrdersCall( apiContext );
            TimeFilter fltr = new TimeFilter();
            fltr.TimeFrom = start;
            fltr.TimeTo = end;

            OrderTypeCollection orders = apicall.GetOrders( fltr, TradingRoleCodeType.Seller, OrderStatusCodeType.All );
            List<EBayOrder> listOrders = new List<EBayOrder>();
            foreach (OrderType order in orders)
            {
                EBayOrder listOrder = new EBayOrder();
                listOrder.OrderID = order.OrderID;
                listOrder.BuyerID = order.BuyerUserID;
                listOrder.CheckoutMessage = order.BuyerCheckoutMessage;
                listOrder.AdjustmentAmount = (decimal) order.AdjustmentAmount.Value;
                listOrder.AmountPaid = (decimal) order.AmountPaid.Value;
                listOrder.AmountSaved = (decimal) order.AmountSaved.Value;
                listOrder.SubTotal = (decimal) order.Subtotal.Value;
                listOrder.Total = (decimal) order.Total.Value;
                listOrder.CreatedTime = order.CreatedTime;
                listOrder.Currency = order.AmountPaid.currencyID.ToString();

                if (order.CheckoutStatus.Status == CompleteStatusCodeType.Complete)
                {
                    listOrder.IsCheckoutCompleted = true;
                }
                else
                {
                    listOrder.IsCheckoutCompleted = false;
                }

                Address address = new Address( order.ShippingAddress.FirstName,
                                order.ShippingAddress.LastName,
                                order.ShippingAddress.CompanyName,
                                order.ShippingAddress.Street1,
                                order.ShippingAddress.Street2,
                                order.ShippingAddress.CityName,
                                order.ShippingAddress.StateOrProvince,
                                order.ShippingAddress.PostalCode,
                                order.ShippingAddress.CountryName,
                                order.ShippingAddress.Phone,
                                order.ShippingAddress.Phone2 );
                ShippingAddress shippingAddress = new ShippingAddress( address, true );
                listOrder.ShippingAddress = shippingAddress;

                List<EBayOrderTransaction> itemTransactions = new List<EBayOrderTransaction>();
                foreach (TransactionType trans in order.TransactionArray)
                {
                    EBayOrderTransaction orderTrans = new EBayOrderTransaction();
                    orderTrans.EBayOrderID = order.OrderID;
                    orderTrans.ItemName = trans.Item.Title;
                    orderTrans.ItemNumber = trans.Item.ItemID;
                    itemTransactions.Add( orderTrans );
                }
                listOrder.ItemTransactions = itemTransactions;

                listOrders.Add( listOrder );
            }

            return listOrders;
        }

        public IList<EBayCategory> GetCategories( string site )
        {
            return GetCategories( "-1", 0, site );
        }

        public IList<EBayCategory> GetCategories( string parentCategoryID, int parentCategoryLevel, string site )
        {
            if (!KeyUtilities.IsDeluxeLicense( DataAccessHelper.DomainRegistrationkey, DataAccessHelper.DomainName ))
            {
                return new List<EBayCategory>();
            }

            ApiContext apiContext = GetApiContext( site );

            StringCollection parentCollection = new StringCollection();
            parentCollection.Add( parentCategoryID );

            DetailLevelCodeTypeCollection detailCollection = new DetailLevelCodeTypeCollection();
            detailCollection.Add( DetailLevelCodeType.ReturnAll );

            GetCategoriesCall categoriesCall = new GetCategoriesCall( apiContext );
            categoriesCall.CategorySiteID = "0";
            categoriesCall.CategoryParent = parentCollection;
            categoriesCall.DetailLevelList = detailCollection;
            categoriesCall.LevelLimit = parentCategoryLevel + 1;
            categoriesCall.ViewAllNodes = true;
            CategoryTypeCollection catCollection = categoriesCall.GetCategories();

            return ConvertCategoryCollectionToList( catCollection );
        }

        public EBayCategory GetCategoriesDetailsByID( string categoryID, string site )
        {
            if (!KeyUtilities.IsDeluxeLicense( DataAccessHelper.DomainRegistrationkey, DataAccessHelper.DomainName ))
            {
                return new EBayCategory();
            }

            ApiContext apiContext = GetApiContext( site );

            StringCollection parentCollection = new StringCollection();
            parentCollection.Add( categoryID );

            DetailLevelCodeTypeCollection detailCollection = new DetailLevelCodeTypeCollection();
            detailCollection.Add( DetailLevelCodeType.ReturnAll );

            GetCategoriesCall categoriesCall = new GetCategoriesCall( apiContext );
            categoriesCall.CategorySiteID = "0";
            categoriesCall.CategoryParent = parentCollection;
            categoriesCall.DetailLevelList = detailCollection;
            //categoriesCall.LevelLimit = parentCategoryLevel + 1;
            categoriesCall.ViewAllNodes = true;
            CategoryTypeCollection catCollection = categoriesCall.GetCategories();
            IList<EBayCategory> catList = ConvertCategoryCollectionToList( catCollection );
            EBayCategory categoryDetail = new EBayCategory();
            foreach (EBayCategory dummyCategory in catList)
            {
                categoryDetail = dummyCategory;
                break;
            }

            return categoryDetail;
        }

        public EBayCategoryFeature GetCategoryFeatureDetail( string parentCategoryID, int parentCategoryLevel, string site )
        {
            if (!KeyUtilities.IsDeluxeLicense( DataAccessHelper.DomainRegistrationkey, DataAccessHelper.DomainName ))
            {
                return new EBayCategoryFeature();
            }

            ApiContext apiContext = GetApiContext( site );
            GetCategoryFeaturesCall featureCall = new GetCategoryFeaturesCall( apiContext );
            featureCall.CategoryID = parentCategoryID;
            featureCall.LevelLimit = parentCategoryLevel;
            featureCall.ViewAllNodes = true;
            featureCall.AllFeaturesForCategory = true;
            DetailLevelCodeTypeCollection detailCollection = new DetailLevelCodeTypeCollection();
            detailCollection.Add( DetailLevelCodeType.ReturnAll );
            featureCall.DetailLevelList = detailCollection;
            CategoryFeatureTypeCollection featureCollectin = featureCall.GetCategoryFeatures();

            IList<EBayCategoryFeature> featureList = ConvertCategoryFeatureCollectionToList( featureCollectin );
            EBayCategoryFeature feature = new EBayCategoryFeature();
            foreach (EBayCategoryFeature dummyFeature in featureList)
            {
                feature = dummyFeature;
            }

            return feature;
        }
        #endregion
    }
}
